﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.TestFunctions.Evaluators;

namespace HeuristicLab.Problems.TestFunctions {
  [Item("CustomAdditiveMoveEvaluator", "Class for evaluating an additive move on a custom test function.")]
  [StorableClass]
  public class CustomAdditiveMoveEvaluator : AdditiveMoveEvaluator {
    public LookupParameter<ISingleObjectiveTestFunctionProblemEvaluator> EvaluatorParameter {
      get { return (LookupParameter<ISingleObjectiveTestFunctionProblemEvaluator>)Parameters["Evaluator"]; }
    }

    [StorableConstructor]
    protected CustomAdditiveMoveEvaluator(bool deserializing) : base(deserializing) { }
    protected CustomAdditiveMoveEvaluator(CustomAdditiveMoveEvaluator original, Cloner cloner) : base(original, cloner) { }
    public CustomAdditiveMoveEvaluator() {
      Parameters.Add(new LookupParameter<ISingleObjectiveTestFunctionProblemEvaluator>("Evaluator", ""));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new CustomAdditiveMoveEvaluator(this, cloner);
    }

    public override System.Type EvaluatorType {
      get { return typeof(CustomEvaluator); }
    }

    protected override double Evaluate(double quality, RealVector point, AdditiveMove move) {
      RealVector vector = new RealVector(point.Select((x, i) => move.Dimension == i ? x + move.MoveDistance : x).ToArray());
      var eval = EvaluatorParameter.ActualValue as CustomEvaluator;
      if (eval != null)
        return eval.Evaluate(vector);
      throw new InvalidOperationException("evaluator is not a custom evaluator");
    }
  }
}
